﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing.Drawing2D;
using System.Drawing;
using System.Windows.Forms;

namespace PaintNew
{
    //sajjad soltani
    //email : s.soltani1989@gmail.com
    //0355503633

    public class PaintClass
    {
        #region Fields...
        private System.Drawing.Graphics _graphic;
        private System.Drawing.Bitmap _bitmap;
        private ActionPaintEnum _actionPaint;
        private System.Drawing.Color _foreColor;
        private int _pensize;
        #endregion

        #region Properties...

        public string TextForm { get; set; }
        
        public MouseEventArgs MouseEvent { get; set; }
        
        public Point StartPoint { get; set; }
        public Point EndPoint { get; set; }
        public Point StartSelectCrop { get; set; }
        public Point EndSelectCrop { get; set; }
        public Color BackColor { get; set; }
        public Font FontForm { get; set; }
        public Pen pen { get; set; }
        public PictureBox PictureBox { get; set;}

        public bool MouseMovement { get; set; }
        public bool FillObject { get; set; }

        public int PenSize { get { return _pensize; } 
                             set { pen.Width = value; _pensize = value; } }

        public ActionPaintEnum ActionPaint
        {
            get { return _actionPaint; }
            set {
                StartPoint = Point.Empty;
                _actionPaint = value; }
        }

        public System.Drawing.Color ForeColor
        {
            get { return _foreColor; }
            set
            {
                if (ChangeForeColor != null)
                    ChangeForeColor(this, new EventChangeColor { OldColor = _foreColor, NewColor = value });

                _foreColor = value;
            }
        }

        #endregion

        #region Constractor...
        public PaintClass(PictureBox _pictureBox)
        {
            PictureBox = _pictureBox;
            NewPaint();
        }
        #endregion

        #region Methods...

        public void NewPaint()
        {
            if (_bitmap!=null)
            {
                _bitmap.Dispose();
                _graphic.Dispose();
            }

            BackColor = Color.White;
            FillObject = false;
            
            pen = new System.Drawing.Pen(Color.Blue,3);
            PenSize = 3;
            StartPoint = EndPoint = System.Drawing.Point.Empty;

            _bitmap = new Bitmap(PictureBox.Width, PictureBox.Height);
            _graphic =    Graphics.FromImage(_bitmap);
            _graphic.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            _graphic.FillRectangle(System.Drawing.Brushes.White, new System.Drawing.Rectangle(0, 0, _bitmap.Width, _bitmap.Height));

            PictureBox.Image = _bitmap;         
        }

        public void OpenFile(string fileString)
        {
            _bitmap.Dispose();
            _graphic.Dispose();

            _bitmap = new Bitmap(Bitmap.FromFile(fileString), PictureBox.Width, PictureBox.Height);
            _graphic =    Graphics.FromImage(_bitmap);
            PictureBox.Image = _bitmap;
        }

        public void MouseMove(MouseEventArgs e)
        {
            MouseEvent = e;
            MouseMovement = true;
            DoAction();
            MouseMovement = false; 
        }

        public void MouseClick(MouseEventArgs e)
        {
            MouseMovement = false;
            MouseEvent = e;
            if (StartPoint.IsEmpty && e.Button == System.Windows.Forms.MouseButtons.Left)
                StartPoint = new Point(e.Location.X, e.Location.Y);
            else
                DoAction();
        }

        private void DoAction()
        {
            if (MouseEvent.Button == MouseButtons.Right)
            {
                StartPoint = Point.Empty;
                PictureBox.Image  = _bitmap;
            }

            EndPoint = new Point(MouseEvent.Location.X,MouseEvent.Location.Y);
            pen.Color = ForeColor;

            try
            {
                switch (ActionPaint)
                {
                    case ActionPaintEnum.Pen:
                        if (MouseEvent.Button == MouseButtons.Left)
                        {
                            SelectTypePen(TypeSelectPen.Draw);
                            Pen(ForeColor);
                        }
                        break;
                    case ActionPaintEnum.Line:
                        if (StartPoint != Point.Empty && EndPoint != Point.Empty)
                        {
                            SelectTypePen(TypeSelectPen.Draw);
                            Line();
                        }
                        break;
                    case ActionPaintEnum.Rectangle:
                        if (StartPoint != Point.Empty && EndPoint != Point.Empty)
                        {
                            SelectTypePen(TypeSelectPen.Draw);
                            Rectangle();
                        }
                        break;
                    case ActionPaintEnum.Circle:
                        if (StartPoint != Point.Empty && EndPoint != Point.Empty)
                        {
                            SelectTypePen(TypeSelectPen.Draw);
                            Circle();
                        }
                        break;
                    case ActionPaintEnum.Text:
                        if (MouseEvent.Button == MouseButtons.Left)
                            Text();
                        break;
                    case ActionPaintEnum.Erase:
                        if (MouseEvent.Button == MouseButtons.Left)
                        {
                            SelectTypePen(TypeSelectPen.Draw);
                            Pen(BackColor);
                        }
                        break;
                    case ActionPaintEnum.Crop:
                        if (StartPoint != Point.Empty && EndPoint != Point.Empty)
                        {
                            SelectTypePen(TypeSelectPen.Crop);
                            Rectangle();
                            if (MouseEvent.Button== MouseButtons.Left)
                            {
                                StartSelectCrop = StartPoint;
                                EndSelectCrop = EndPoint;
                                Crop();
                                StartPoint = Point.Empty;                   
                            }     
                        }
                        break;

                    default:
                        MessageBox.Show("Not Select Action");
                        break;
                }
            }
            catch (Exception)
            {
                throw;
            }
        }

        private void Text()
        {
            TextForm frm = new TextForm();
            frm.ShowDialog();           
        }

        private void Rectangle()
        {
            Bitmap _tmpBitmap;
            Graphics _g;

            if (MouseMovement)
            {
                _tmpBitmap = (Bitmap)_bitmap.Clone();
                _g = Graphics.FromImage(_tmpBitmap) ;
            }
            else
            {
                _tmpBitmap = _bitmap;
                _g = _graphic;
            }

            if (FillObject)
            {
                LinearGradientBrush linearBr = new LinearGradientBrush(new Rectangle(0, 0, 4, 4), ForeColor, ForeColor, LinearGradientMode.BackwardDiagonal);
                _g.FillRectangle(linearBr, StartPoint.X, StartPoint.Y, Math.Abs(StartPoint.X - EndPoint.X), Math.Abs(StartPoint.Y - EndPoint.Y));
            }
            else
                _g.DrawRectangle(pen, StartPoint.X, StartPoint.Y, Math.Abs(StartPoint.X - EndPoint.X), Math.Abs(StartPoint.Y - EndPoint.Y));

            PictureBox.Image = _tmpBitmap;
        }

        private void Circle()
        {
            int radius = Math.Max(Math.Abs(StartPoint.X - EndPoint.X), Math.Abs(StartPoint.Y - EndPoint.Y));
            Bitmap _tmpBitmap;
            Graphics _g;

            if (MouseMovement)
            {
                 _tmpBitmap =(Bitmap) _bitmap.Clone();
                 _g = Graphics.FromImage(_tmpBitmap);
            }
            else
            {
                _tmpBitmap = _bitmap;
                _g = _graphic;
            }

            if (FillObject)
            {
                LinearGradientBrush linearBr = new LinearGradientBrush(new Rectangle(0, 0, 4, 4), ForeColor, ForeColor, LinearGradientMode.BackwardDiagonal);
                _g.FillEllipse(linearBr, (float)StartPoint.X - (radius), (float)StartPoint.Y - (radius), radius * 2, radius * 2);
            }
            else
                _g.DrawEllipse(pen, (float)StartPoint.X - (radius), (float)StartPoint.Y - (radius), radius * 2, radius * 2);
            
            PictureBox.Image = _tmpBitmap;

            if (!MouseMovement)
                StartPoint = Point.Empty;
        }

        private void Line()
        {
            Bitmap _tmpBitmap;
            Graphics g;

            if (MouseMovement)
            {
                _tmpBitmap = (Bitmap)_bitmap.Clone();
                g = Graphics.FromImage(_tmpBitmap);
            }
            else
            {
                _tmpBitmap = _bitmap;
                g = _graphic;
            }

            g.DrawLine(pen, StartPoint, EndPoint);
            PictureBox.Image = _tmpBitmap;
        }

        private void Pen(System.Drawing.Color color)
        {
            //LinearGradientBrush linearBr = new LinearGradientBrush(new Rectangle(0, 0, 4, 4), color, color, LinearGradientMode.ForwardDiagonal);
            //_graphic.FillEllipse(linearBr, EndPoint.X - (pen.Width / 2), EndPoint.Y - (pen.Width / 2), pen.Width, pen.Width);
            _graphic.FillEllipse(new SolidBrush(ForeColor) , EndPoint.X - (pen.Width / 2), EndPoint.Y - (pen.Width / 2), pen.Width, pen.Width);
            PictureBox.Image = _bitmap;
        }

        public void ImageGrayScale()
        {
            System.Drawing.Imaging.ColorMatrix colormatrix = new System.Drawing.Imaging.ColorMatrix(
                new float[][]{new float[] {1,0,0,0,0},
                              new float[] {0,0,1,0,0},
                              new float[] {0,0,0,0,0},
                              new float[] {0,0,0,0,0},
                              new float[] {0,1,0,0,0}});


            System.Drawing.Imaging.ImageAttributes attribute = new System.Drawing.Imaging.ImageAttributes();
            attribute.SetColorMatrix(colormatrix);

            _graphic.DrawImage(_bitmap, new Rectangle(0, 0, _bitmap.Width, _bitmap.Height), 0, 0, _bitmap.Width, _bitmap.Height, GraphicsUnit.Pixel, attribute);
            PictureBox.Image = _bitmap;
            PictureBox.Refresh();
        }
       
        public void Rotate(int rotate)
        {
            _graphic.TranslateTransform((float)_bitmap.Width / 2, (float)_bitmap.Height / 2);
            _graphic.RotateTransform(rotate);
            _graphic.TranslateTransform(-(float)_bitmap.Width / 2, -(float)_bitmap.Height / 2);
            _graphic.DrawImage(_bitmap, new Point(0, 0));
            PictureBox.Image = _bitmap;
        }

        private void Crop()
        {
            _bitmap = (Bitmap)_bitmap.Clone(new RectangleF((float)StartSelectCrop.X + 1, (float)StartSelectCrop.Y + 1, (float)EndSelectCrop.X - StartSelectCrop.X - 1, (float)EndSelectCrop.Y - StartSelectCrop.Y - 1), _bitmap.PixelFormat);
            _graphic = Graphics.FromImage(_bitmap);
            PictureBox.Image = _bitmap;
            PictureBox.Width = _bitmap.Width;
            PictureBox.Height = _bitmap.Height;
            PictureBox.Refresh();
        }

        private void SelectTypePen(TypeSelectPen sel)
        {
            switch (sel)
            {
                case TypeSelectPen.Crop:
                    pen.DashStyle = DashStyle.Dash;
                    pen.Color = Color.Black;
                    pen.Width = 1;
                    break;
                case TypeSelectPen.Draw:
                    pen.DashStyle = DashStyle.Solid;
                    pen.Color = ForeColor;
                    pen.Width = PenSize;
                    break;
                default:
                    break;
            }
        }

        #endregion

        #region Events...
        public event ChangeForeColorDelegate ChangeForeColor;
        #endregion

    }
}
